YUI().use(
  'gallery-base64', 'node', 'history', 'io', 'cookie', 'querystring-stringify-simple', 'json', 'linaro-overlay-utils', 'linaro-trivia',
  function (Y) {
    var history = new Y.HistoryHash();

    Y.on(
      'history:change',
      function (e) {
        if (e.src === Y.HistoryHash.SRC_HASH) {
          displayJobByNumber(e.changed.build.newVal);
        }
      }
    );

    var allBuilds;
    var buildNumber = history.get('build');
    var fetchLatest = (buildNumber === undefined);

    var jobData = 'actions[parameters[name,value]],timestamp,duration,artifacts[displayPath,fileName,relativePath],result,number';

    var buildName = rawBuildName.replace('/', '_').substring(1);

    var treeExp = 'actions[parameterDefinitions[defaultParameterValue[value],name]],builds[number],queueItem[params]';
    if (fetchLatest) {
      treeExp += ',lastBuild[' + jobData + ']';
    }
    var jobUrl = globalConfig.jenkinsURL + "/job/" + buildName;

    var element_mapping = {
      status: function (results) {
        return results.result || "RUNNING";
      },
      configuration: function (results) {
        if (results.actions[0].parameters) {
          var params = results.actions[0].parameters;
          for (var i = 0; i < params.length; i++) {
            if (params[i].name == "CONFIG") {
              try {
                return Y.Base64.decode(params[i].value);
              } catch (error) {
                return 'Not Base64 encoded?';
              }
            }
          }
        }
        return '???';
      },
      started: function (results) {
        return Y.linaro.formatDateFromTimestamp(results.timestamp);
      },
      finished: function (results) {
        if (results.timestamp && results.duration) {
          return Y.linaro.formatDateFromTimestamp(results.timestamp + results.duration);
        } else {
          return '';
        }
      },
      loglink: function (results) {
        return Y.Node.create('<a>complete output</a>').setAttribute(
          'href', results.buildUrl + '/consoleText');
      },
      output: function (results) {
        if (results.artifacts.length) {
          Y.one('#all-output a').setAttribute(
            'href', results.number.toString() + '/output/');
          Y.one('#all-output a').removeClass('hidden');
          Y.one('#all-output span').addClass('hidden');
          var listNode = Y.Node.create('<ul/>');
          for (var i = 0; i < results.artifacts.length; i++) {
            var artifact = results.artifacts[i];
            var itemNode = Y.Node.create('<li><a/></li>');
            itemNode.one('a').setContent(artifact.displayPath);
            itemNode.one('a').setAttribute('href', results.buildUrl + '/artifact/' + artifact.relativePath);
            listNode.appendChild(itemNode);
          }
          return listNode;
        } else {
          Y.one('#all-output a').addClass('hidden');
          Y.one('#all-output span').removeClass('hidden');
          return Y.Node.create("<p>No output</p>");
        }
      }
    };

    function displayBuild (results) {
      results.buildUrl = jobUrl + '/' + results.number.toString();
      for (var id in element_mapping) {
        var val = element_mapping[id](results);
        var targetNode = Y.one('#' + id);
        if (Y.Lang.isString(val)) {
          targetNode.setContent(val);
        } else {
          targetNode.get('childNodes').remove();
          targetNode.appendChild(val);
        }
      }
      Y.one("#olderlinks").get('childNodes').remove();
      Y.one("#olderlinks").appendChild(olderlinks(results.number));
    }

    function olderlinks (curNumber) {
      var resultNode = Y.Node.create('<span/>');
      for (var i = 0; i < allBuilds.length; i++) {
        var node = Y.Node.create('<span />');
        var num = allBuilds[i].number;
        node.setAttribute('id', 'oldlink-' + i.toString());
        if (num != curNumber) {
          var linkNode = Y.Node.create('<a/>');
          linkNode.setAttribute('href', '#build=' + num.toString());
          linkNode.setContent(num.toString());
          node.appendChild(linkNode);
        } else {
          node.append(curNumber.toString());
        }
        resultNode.append(node);
        resultNode.append(document.createTextNode(' '));
      }
      return resultNode;
    }

    var buildConfig;

    function refreshFromScratch() {
      Y.io(
        '../../../api/is-daily?build=' + rawBuildName,
        {
          on: {
            success: function (e, response) {
              var result = Y.JSON.parse(response.responseText);
              if (result) {
                Y.one('#not-built-daily').addClass('hidden');
              } else {
                Y.one('#not-built-daily').removeClass('hidden');
              }
              Y.one('#built-daily-details').removeClass('hidden');
            }
          }
        }
      );

      Y.io(
        jobUrl + "/api/json?tree=" + treeExp,
        {
          on: {
            success: function (e, response) {
              var job = Y.JSON.parse(response.responseText);
              Y.all('.build-found').removeClass('hidden');
              Y.Array.each(
                job.actions[0].parameterDefinitions,
                function (defn) {
                  if (defn.name == "CONFIG") {
                    try {
                      buildConfig = Y.Base64.decode(defn.defaultParameterValue.value);
                    } catch (error) {
                      buildConfig = '';
                    }
                  }
                });

              if (job.queueItem) {
                Y.one('#queue-details').removeClass('hidden');
                var queueConfig;
                var encodedConfig = job.queueItem.params.replace(/^[\s\S]*CONFIG='([\s\S]*)'$/,'$1');
                try {
                  queueConfig= Y.Base64.decode(encodedConfig);
                } catch (error) {
                  queueConfig = 'Not base64 encoded?';
                }
                Y.one("#queued-configuration").setContent(queueConfig);
              }

              if (history.get('editing')) { beginEditing(); }
              if (job.builds.length) {
                Y.one("#build-details").setStyle("display", "block");
                allBuilds = job.builds;
                if (fetchLatest) {
                  displayBuild(job.lastBuild);
                } else {
                  displayJobByNumber(buildNumber);
                }
              } else {
                Y.one("#no-build").setStyle("display", "block");
              }
            },
            failure: function (e) {
              Y.all('.build-found').addClass('hidden');
              Y.one("#build-not-found").setStyle("display", "block");
            }
          }
        }
      );
    }

    refreshFromScratch();

    function displayJobByNumber(num) {
      Y.io(
        jobUrl + "/" + num.toString() + "/api/json?tree=" + jobData,
        {
          on: {
            success: function (e, response) {
              displayBuild(Y.JSON.parse(response.responseText));
            },
            failure: function (e) {
              Y.all('.build-found').addClass('hidden');
              Y.one("#build-not-found").setStyle("display", "block");
            }
          }
        }
      );
    }

    Y.io.header('X-CSRFToken', Y.Cookie.get("csrftoken"));

    Y.on(
      'click',
      function (e) {
        var buildNow = Y.one("#build-now");
        buildNow.set('disabled', true);
        Y.io(
          '../../../api/build-now',
          {
            method: "POST",
            data: {
              build: rawBuildName
            },
            on: {
              failure: function (e, response) {
                Y.linaro.reportError(response.responseText, {code:response.status});
              },
              complete: function (e, response) {
                buildNow.set('disabled', false);
              },
              success: function (e, response) {
                buildNumber = history.get('build');
                fetchLatest = (buildNumber === undefined);
                refreshFromScratch();
              }
            }
          }
        );
      },
      '#build-now');

    Y.on(
      'click',
      function (e) {
        var isDailyCurrently = Y.one('#not-built-daily').hasClass('hidden');
        var toggleDaily = Y.one('#toggle-daily');

        toggleDaily.set('disabled', true);

        Y.io(
          '../../../api/set-daily',
          {
            method: "POST",
            data: {
              build: rawBuildName,
              daily: !isDailyCurrently
            },
            on: {
              failure: function (e, response) {
                Y.linaro.reportError(response.statusText);
              },
              complete: function (e, response) {
                toggleDaily.set('disabled', false);
              },
              success: function (e, response) {
                refreshFromScratch();
              }
            }
          }
        );
      },
      '#toggle-daily');

    var reallyDeleteOverlay = Y.linaro.makeYesNoOverlay(
      '#really-delete',
      {
        yes: function () {
          Y.io(
            '../../../api/delete',
            {
              method: "POST",
              data: {
                build: rawBuildName
              },
              on: {
                failure: function (e, response) {
                  Y.linaro.reportError(response.statusText);
                },
                complete: function (e, response) {
                  reallyDeleteOverlay.hide();
                },
                success: function (e, response) {
                  // could go back to official or mine or all as appropriate here
                  window.location.hash = '';
                  window.location.pathname += '../../..';
                }
              }
            }
          );
        }
      }
    );

    Y.on(
      'click',
      function (e) {
        reallyDeleteOverlay.show();
      },
      "#delete");

    var editOverlay = Y.linaro.makeYesNoOverlay(
      "#edit-overlay",
      {
        width: '80ex',
        height: '40ex',
        yes: function () {
          var newText = editOverlay.get('srcNode').one('textarea').get('value');
          Y.io(
            '../../../api/edit',
            {
              method: "POST",
              data: {
                build: rawBuildName,
                newText: newText
              },
              on: {
                failure: function (e, response) {
                  Y.linaro.reportError(response.statusText);
                },
                complete: function (e, response) {
                  editOverlay.hide();
                },
                success: function (e, response) {
                  buildConfig = newText;
                }
              }
            }
          );
        }
      }
    );

    editOverlay.on(
      'visibleChange',
      function (e) {
        if (e.newVal) {
          history.addValue('editing', 1);
        } else {
          history.addValue('editing', null);
        }
      }
    );

    function beginEditing () {
      var textarea = editOverlay.getStdModNode(Y.WidgetStdMod.BODY).one('textarea');
      textarea.set('value', buildConfig);
      textarea.setStyles(
        { // XXX hand chosen fudge factors here...
          width: textarea.ancestor().get('offsetWidth') - 28,
          height: textarea.ancestor().get('offsetHeight') - 28
        });
      editOverlay.show();
    }
    Y.on('click', beginEditing, "#edit");
  }
);
