YUI.add(
  'linaro-overlay-utils',
  function(Y) {

    Y.namespace('linaro');

    Y.linaro.makeOverlay = function (srcNode, cfg) {

      cfg = cfg || {};
      var overlay = new Y.Overlay(
        {
          srcNode: srcNode,
          width: cfg.width || '400px',
          height: cfg.height || '200px',
          zIndex: 100,
          constrain: true,
          centered: true,
          render: cfg.render,
          visible: false,
          plugins: [
            Y.Plugin.OverlayModal,
            Y.Plugin.OverlayKeepaligned,
            Y.Plugin.OverlayAutohide
          ]
        }
      );
      if (!cfg.render)
        Y.on('domready', function () { overlay.render(); });
      cfg.hide_button_class &&
        overlay.get('srcNode').all('.' + cfg.hide_button_class).on('click', Y.bind(overlay.hide, overlay));
      return overlay;
    };

    var _errorOverlay = null;

    function getErrorOverlay () {
      if (_errorOverlay) return _errorOverlay;
      _errorOverlay = Y.linaro.makeOverlay(
        '#generic-error', {render: true, hide_button_class: 'hide'});
      return _errorOverlay;
    }

    Y.linaro.reportError = function (error, cfg) {
      cfg = cfg || {};
      var errorOverlay = getErrorOverlay();

      errorOverlay.set('bodyContent', error);

      var header = errorOverlay.getStdModNode(Y.WidgetStdMod.HEADER).one('h3');

      if (cfg.header) {
        header.setContent(cfg.header);
      } else if (cfg.code) {
        header.setContent("Error " + cfg.code.toString());
      } else {
        header.setContent("Error");
      }

      errorOverlay.show();
    };

    Y.linaro.makeYesNoOverlay = function (srcNode, cfg) {
      cfg = cfg || {};
      cfg.hide_button_class = cfg.hide_button_class || 'no';
      var overlay = new Y.linaro.makeOverlay(srcNode, cfg);

      overlay.after(
        'render',
        function (e) {
          var overlayBody = overlay.getStdModNode(Y.WidgetStdMod.BODY);
          overlay.subOverlay = new Y.Overlay(
            {
              bodyContent:'<div class="trans" style="height: 100%"></div>',
              centered: overlayBody,
              height: overlayBody.get('offsetHeight'),
              width: overlayBody.get('offsetWidth'),
              visible: false,
              render: overlayBody
            }
          );
          var subOverlayBody = overlay.subOverlay.getStdModNode(Y.WidgetStdMod.BODY);
          var subsubOverlay = new Y.Overlay(
            {
              bodyContent:'<img src="/static/spinner.gif"/>',
              centered: subOverlayBody,
              height: '32px',
              width: '32px',
              visible: true,
              render: subOverlayBody
            }
          );
        }
      );

      overlay.on(
        'visibleChange',
        function (e) {
          if (e.newVal) {
            overlay.get('srcNode').all('button').set("disabled", false);
            overlay.plug(Y.Plugin.OverlayAutohide);
          } else {
            overlay.subOverlay.hide();
          }
        }
      );

      overlay.get('srcNode').one('.yes').on(
        'click',
        function (e) {
          cfg.yes();
          overlay.get('srcNode').all('button').set("disabled", true);
          overlay.unplug(Y.Plugin.OverlayAutohide);
          overlay.subOverlay.show();
        });

      return overlay;
    };

  },
  '0.0.1' /* module version */,
  {
    gallery: 'gallery-2011.02.16-20-31',
    requires: ['overlay', 'gallery-overlay-extras']
  }
);

YUI.add(
  'linaro-trivia',
  function(Y) {

    Y.namespace('linaro');

    Y.linaro.formatDateFromTimestamp = function (timestamp) {
      return Y.DataType.Date.format(new Date(timestamp), {format: "%Y-%m-%d %T"});
    };
  },
  '0.9.1' /* module version */,
  {
    requires: ['datatype-date']
  }
);
